/*
 *  1541t64.h - 1541 emulation in .t64/LYNX file
 *
 *  Frodo (C) 1994-1997,2002 Christian Bauer
 */

#ifndef _1541T64_H
#define _1541T64_H

#include "IEC.h"

class C64;

// An arbitary large number (> max numbers of file in a t64)
#define IS_DIR 1000000

// Information for file inside a .t64 file
typedef struct {
	char name[17];		// File name, PETSCII
	uint8 type;		// File type
	uint8 sa_lo, sa_hi;	// Start address
	int offset;		// Offset of first byte in .t64 file
	int length;		// Length of file
} FileInfo;


class T64Drive : public Drive {
public:
	T64Drive(C64 *c64, IEC *iec, FILE *f);
	~T64Drive();
	uint8 Open(int channel, char *filename);
	uint8 Close(int channel);
	uint8 Read(int channel, uint8 *byte);
	uint8 Write(int channel, uint8 byte, bool eoi);
	void Reset(void);

private:
	void open_close_t64_file(FILE *t64);
	uint8 read_directory(int channel, uint8 *b);
	bool parse_t64_file(void);
	bool parse_lynx_file(void);
	uint8 open_file(int channel, char *filename);
	uint8 open_directory(int channel, char *filename);
	void convert_filename(char *srcname, char *destname, int *filemode, int *filetype);
	bool find_first_file(char *name, int type, int *num);
	void close_all_channels(void);
	void execute_command(char *command);
	void cht64_cmd(char *t64path);
	uint8 conv_from_64(uint8 c);

	FILE *the_file;			// File pointer for .t64 file

	char dir_title[16];		// Directory title
	int32 file_num[16];		// File id for an open channel
	int32 file_offs[16];		// Bytes read for an open channel
	uint32 *dir_matches[16];	// File id's for matches in load "$"
	int32 dir_nmatch[16];		// Length of above list
	uint8 dir_buffer[16][32];	// Buffer for dir search on a channel

	int num_files;			// Number of files in .t64 file and in file_info array
	FileInfo *file_info;	// Pointer to array of file information structs for each file

	char cmd_buffer[44];	// Buffer for incoming command strings
	int cmd_len;			// Length of received command

	uint8 read_char[16];	// Buffers for one-byte read-ahead

	bool is_lynx;			// Flag: .t64 file is really a LYNX archive
};

#endif
