/*
 *  C64.h - Put the pieces together
 *
 *  Frodo (C) 1994-1997,2002 Christian Bauer
 */

#ifndef _C64_H
#define _C64_H

#include "palmposix.h"
#include "Prefs.h"

class C64Display;
class MOS6510;
class MOS6569;
class MOS6581;
class MOS6526_1;
class MOS6526_2;
class IEC;
class MOS6502_1541;
class Job1541;
class CmdPipe;

class C64 {
public:
	C64(ArmletData *);
	~C64();

	void Run(void);
	void Quit(void);
	void Pause(void);
	void Resume(void);
	void Reset(void);
	void NMI(void);
	void VBlank(bool draw_frame);
	void NewPrefs(Prefs *prefs);
	void PatchKernal(bool fast_reset, bool emul_1541_proc);
	void SaveSnapshot(uint8 *block1, uint8 *block2);
	bool LoadSnapshot(uint8 *block1, uint8 *block2);
	uint32 SaveCPUState(uint8 *p1, uint8 *p2);
	uint32 Save1541State(uint8 *p);
	uint16 Save1541JobState(uint8 *p);
	uint16 SaveVICState(uint8 *p);
	uint16 SaveSIDState(uint8 *p);
	uint16 SaveCIAState(uint8 *p);
	uint16 LoadCPUState(uint8 *p1, uint8 *p2);
	uint16 Load1541State(uint8 *p);
	uint16 Load1541JobState(uint8 *p);
	uint16 LoadVICState(uint8 *p);
	uint16 LoadSIDState(uint8 *p);
	uint16 LoadCIAState(uint8 *p);

	uint16 LoadCPUStateOld(uint8 *p1, uint8 *p2);
	uint16 Load1541StateOld(uint8 *p);
	uint16 LoadVICStateOld(uint8 *p);
	uint16 LoadCIAStateOld(uint8 *p);

	uint8 *RAM;
	uint8 *Basic, *Kernal, *Char, *Color;		// C64
  uint8 *IO_Ram; // Simulate RAM if jmp $dxxxx occurs
	uint8 *RAM1541, *ROM1541;	// 1541

	C64Display *TheDisplay;

	MOS6510 *TheCPU;			// C64
	MOS6569 *TheVIC;
	MOS6581 *TheSID;
	MOS6526_1 *TheCIA1;
	MOS6526_2 *TheCIA2;
	IEC *TheIEC;

	MOS6502_1541 *TheCPU1541;	// 1541
	Job1541 *TheJob1541;

	uint32 CycleCounter;

private:
	void c64_ctor1(void);
	uint8 poll_joystick(int port);
	void thread_func(void);

	bool thread_running;	// Emulation thread is running
	bool quit_thyself;		// Emulation thread shall quit
	bool have_a_break;		// Emulation thread shall pause

	int joy_minx, joy_maxx, joy_miny, joy_maxy; // For dynamic joystick calibration

	uint8 orig_kernal_1d84,	// Original contents of kernal locations $1d84 and $1d85
		  orig_kernal_1d85;	// (for undoing the Fast Reset patch)

public:
	ArmletData *armletData;
	Prefs ThePrefs;
	Prefs TheNewPrefs;
	uint8 Random();
	void SeedRandom(uint32);
private:
	uint32 seed;
	bool SwitchToSC;
	bool SwitchToStandard;
};

#endif
