/*
 *  Display.cpp - C64 graphics display, emulator window handling
 *
 *  Frodo (C) 1994-1997,2002 Christian Bauer
 */

#include "sysdeps.h"

#include "Display.h"
#include "main.h"
#include "Prefs.h"
#include "C64.h"
#include "Version.h"


/*
 *  Update drive LED display (deferred until Update())
 */

void C64Display::UpdateLEDs(uint8 led)
{
	TheC64->armletData->driveLED = led;
}


/*
 *  Display constructor: Create window/screen
 */

C64Display::C64Display(C64 *the_c64) : TheC64(the_c64)
{
	// Everything has already been set up by the 68k front end
}


/*
 *  Display destructor
 */

C64Display::~C64Display()
{
}


/*
 *  Return pointer to bitmap data
 */

uint8 *C64Display::BitmapBase(void)
{
	return TheC64->armletData->displayBuf;
}


/*
 *  Return number of bytes per row
 */

int C64Display::BitmapXMod(void)
{
	return TheC64->armletData->displayModulo;
}


/*
 *  Handle IDCMP messages
 */

void C64Display::PollKeyboard(uint8 *key_matrix, uint8 *rev_matrix)
{
	// The key in armletData is simply the key number left to right, top to bottom
	// Convert to C64 row/column
	
	uint32 direction = (TheC64->armletData->key>>16) & 0xff;
	int8 c64_byte = (TheC64->armletData->key>>8) & 0xff;
	int8 c64_bit = TheC64->armletData->key & 0xff;

	if (c64_byte != -1)
	{
		// Normal keys
		bool shifted = c64_byte & 8;
		c64_byte &= 7;
		if (direction == KEYDOWN) 
		{
			// Key pressed
			if (shifted) 
			{
				key_matrix[6] &= 0xef;
				rev_matrix[4] &= 0xbf;
			}
			key_matrix[c64_byte] &= ~(1 << c64_bit);
			rev_matrix[c64_bit] &= ~(1 << c64_byte);
		} 
		else if (direction == KEYUP) 
		{
			// Key released
			if (shifted) 
			{
				key_matrix[6] |= 0x10;
				rev_matrix[4] |= 0x40;
			}
			key_matrix[c64_byte] |= (1 << c64_bit);
			rev_matrix[c64_bit] |= (1 << c64_byte);
		}
	}
}


/*
 *  Show a requester
 */

int32 ShowRequester(int32 id, int32 arg1, int32 arg2, int32 arg3, int32 arg4)
{
	Int32 args[5]={id, arg1, arg2, arg3, arg4};
	Int32 *argp=&(args[0]);
	ArmletData *ad=(ArmletData *)GetR10();
	UInt32 retval;

	PutR10(ad->r10org);
  ad->r9curr=GetR9();
	PutR9(ad->r9org);
	retval = ad->call68k(ad->emulStateP, (uint32)ad->requester, &argp, 4);
	ad->r10org=GetR10();
	PutR10((uint32)ad);
	ad->r9org=GetR9();
  PutR9(ad->r9curr);

	return retval;
}
