//
//  D64Handler.java
//  C64Palmer
//
//  Created by Johan Forsberg on Thu Aug 28 2003.
//  Copyright (c) 2003 __MyCompanyName__. All rights reserved.
//

import java.io.*;

public class D64Handler {
    Byte d[];
    T64Handler t64=new T64Handler();
    
    public PDBItem handle(File f) {
        if (f==null) return null;
        
        byte b[]=new byte[(int)f.length()];
        try {
            FileInputStream fis=new FileInputStream(f);
            int i=fis.read(b);
            if (i!=f.length())
                throw new Exception("Wrong file size");
        } catch (Exception x) {
            x.printStackTrace();
            return null;
        }
        return handle(f.getName(), b);
    }
    /* A .d64 is a byte-by-byte copy of an 1541 disk, optionally followed by a
       bad block list.
       Disk sectors are 256 bytes. The different tracks contain different numbers
       of sectors. Tracks are counted from 1 and up while sectors in a track are
       counted from 0 and up.
       On a standard disk there's a BAM (block allocation map) at 18/0. The first
       2 bytes are the track/sector of the first directory block. HOWEVER, this
       information should not be trusted (a real 1541 ignores it). Always look
       for directory block 1 at 18/1.
       Each directory block can contain 8 entries of 32 bytes. The first two bytes
       of the first entry is a link to the next directory block, or 0/* to
       end the directory.
       The third byte of the entry is the type. 0=scratch (unused or deleted),
       130 is PRG.
       As for files, the first two bytes of each sector is a link (track/sector)
       to the next sector, the other 254 bytes are content. A zero in the track
       link means that this is the last sector, and the sector part of the link
       indicates number of bytes of data in this sector.
    */
    static int track2sector[] = {
        0,21,42,63,84,105,126,147,168,189,210,231,252,273,294,315,336, // 21 s/t
        357,376,395,414,433,452,471,                                   // 19 s/t
        490,508,526,544,562,580,                                       // 18 s/t
        598,615,632,649,666,683,700,717,734,751                        // 17 s/t
    };
    public PDBItem handle(String name, byte data[]) {
        //int BAM=(track2sector[17])*256;
        //int dir=(track2sector[data[BAM]]+data[BAM+1])*256;
        int dir=(track2sector[17]+1)*256;
        
        if (data[dir]==0) {
            // next directory track==0 -> last directory block
            // This is a likely canidate. We should have one PRG and 7 unused files
            byte b[]=new byte[8];
	    for (int i=0; i<8; i++)
		b[i]=data[dir+2+i*32];
	    if (data[dir+2]==((byte)0x82) && // type==PRG
                data[dir+34]==0 && data[dir+66]==0 && data[dir+98]==0 &&
                data[dir+130]==0 && data[dir+162]==0 && data[dir+194]==0 &&
                data[dir+226]==0) {

                // The C=64 file name is 16 bytes at 0x05 to 0x14
                byte c64name[]=new byte[16];
                System.arraycopy(data, dir+5, c64name, 0, 16);
                
                // This seems to be a .t64/.p00/whatever converted to a d64.
                // Convert it to a simple data file.
                int p=(track2sector[data[dir+3]-1]+data[dir+4])*256;
                ByteArrayOutputStream baos=new ByteArrayOutputStream();
                while (data[p]!=0) {
                    baos.write(data, p+2, 254);
                    p=(track2sector[data[p]-1]+data[p+1])*256;
                }
		// Final sector, second byte indicates data length.
                baos.write(data, p+2, ((int)data[p+1])&0xff);
                   
                // Then make it into a t64.
                return t64.handlePRG(name, c64name, baos.toByteArray());
                
            }
        }
        return new PDBItem(name, data);
    }
}
