#include "palmposix.h"

#include "Display.h"

#define kPceNativeWantA0                   (0x10000000)


// No fopen. Files appear magically, delivered by the 68k

UInt32 fread(void *buf, UInt32 size, UInt32 n, FILE *f)
{
	uint32 i, rd;
	int32 retval=0, chunksz, ok;
	uint8 *b=(uint8 *) buf;

	for (i=0; i<n; i++)
	{
		// Read past end
		if (f->offset + size > f->size)
			return retval;
		// Chunk boundrary (size may be many times partSize)
		if (f->partOffset + size > f->partSize)
		{
			rd=0;
			while (rd < size)
			{
				chunksz = f->partSize - f->partOffset;
				if (chunksz > (int32) (size - rd))
					chunksz = size - rd;
				ok = fread(b, chunksz, 1, f);
				if (!ok) 
				  return retval;
				rd += chunksz;
				b += chunksz;
			}
			retval++;
			continue;
		}
		// All chunk crossings taken care of above
		memcpy(b, f->part[f->currentPart].p + f->partOffset, size);
		retval++;
		b += size;
		f->offset += size;
		f->partOffset += size;
		if (f->partOffset == f->partSize)
		{
			f->partOffset=0;
			f->currentPart++;
		}
	}
	return retval;
}

UInt32 fwrite(const void *buf, UInt32 size, UInt32 n, FILE *f)
{
	uint32 i, wr;
	int32 retval=0, chunksz, ok;
	const uint8 *b=(const uint8 *) buf;
  uint8 *dst;
  
	for (i=0; i<n; i++)
	{
		// Write past end, not allowed
		if (f->offset + size > f->size)
			return retval;
		// Chunk boundrary (size may be many times partSize)
		if (f->partOffset + size > f->partSize)
		{
			wr=0;
			while (wr < size)
			{
				chunksz=f->partSize;
				if (chunksz > (int32) (size-wr))
				  chunksz = size - wr;
				ok = fwrite(b, chunksz, 1, f);
				if (!ok)
				  return retval;
				wr+=chunksz;
				b+=chunksz;
			}
			retval++;
			continue;
		}
		// All chunk crossings taken care of above
		dst = f->part[f->currentPart].p + f->partOffset;

		if(f->location != 0L && f->part[0].id == 0xffff0000)
		{
      // File loaded from expansion card into memory -> direct write access
		  memcpy(f->part[f->currentPart].p + f->partOffset, b, size);
		}
		else
    {
  		// We must use DmWrite to write to memory of database entries
    	ArmletData *ad=(ArmletData *)GetR10();
    	Int32 retval;
      Int32 args[4]={(Int32) f->part[f->currentPart].p, f->partOffset, (Int32) b, size};
    	Int32 *argp=&(args[0]);
    	
    	PutR10(ad->r10org);
    	ad->r9curr=GetR9();
    	PutR9(ad->r9org);
    	retval = ad->call68k(ad->emulStateP, (uint32) ad->dmwrite, &argp, 4);
    	ad->r10org=GetR10();
    	PutR10((uint32)ad);
    	ad->r9org=GetR9();
    	PutR9(ad->r9curr);
    } 		  
		retval += size;
		b += size;
		f->offset += size;
		f->partOffset += size;
		if (f->partOffset == f->partSize)
		{
			f->partOffset=0;
			f->currentPart++;
		}
	}
	return retval;
}

int16 fgetc(FILE *f)
{
	int16 retval;

	if (f->offset >= f->size)
		return EOF;

	retval = f->part[f->currentPart].p[f->partOffset] & 0xff;
	f->partOffset++;
	f->offset++;
	if (f->partOffset == f->partSize)
	{
		f->partOffset=0;
		f->currentPart++;
	}

	return retval;
}

UInt32 feof(FILE *f)
{
	return f->offset >= f->size;
}

Int32 fseek(FILE *f, Int32 n, UInt32 mode)
{
	switch (mode)
	{
		case SEEK_CUR:
		  return fseek(f, f->offset+n, SEEK_SET);
		case SEEK_END:
		  return fseek(f, f->size+n, SEEK_SET);
		case SEEK_SET:
		  break;
		default:
		  return -1;
	}

	// SEEK_SET it is.
	if (n> (Int32) (f->size) || n<0)
	  return -1;

	f->offset=n;
	f->currentPart=0;
	for (; n >= (int32) (f->partSize); n-=f->partSize)
	{
		f->currentPart++;
	}
	f->partOffset=n;
	
	return 0;
}

UInt32 ftell(FILE *f)
{
	return f->offset;
}

void fclose(FILE *f)
{
}

// Memory and string functions (now handled in assembler)
/*
void *memcpy(void *dst, const void *src, uint32 n)
{
	uint32 i;
  
	if (!src || !dst) return dst;
	for (i=0; i<n; i++)
		((char *)dst)[i]=((char *)src)[i];

	return dst;
}
*/
/*
void *memset(void *dst, int8 val, uint32 n)
{
	uint32 i;
	char *p=(char *)dst;

	if (!dst) return dst;
	for (i=0; i<n; i++)
		*p++ = val;

	return dst;
}
*/

char *strchr(char *src, const char c)
{
	char *retval;

	if (!src) 
	  return NULL;
	for (retval=src; *retval; retval++)
	{
		if (*retval == c)
			return retval;
  }
  
	return NULL;
}

char *strncpy(char *dst, const char *src, uint32 n)
{
	uint32 i;

	if (!src || !dst)
	  return dst;
	for (i=0; i<n; i++)
	{
		dst[i]=src[i];
		if (dst[i] == '\0')
			break;
	}
	return dst;
}

char *strcpy(char *dst, const char *src)
{
	uint32 i;
	
	if (!src || !dst)
	  return dst;
	for(i=0; src[i] != '\0'; ++i)
	  dst[i]=src[i];
	dst[i]='\0';

	return dst;
}

int16 strcmp(const char *a, const char *b)
{
	if (!a && !b)
	  return 0;
	if (!a) 
	  return -1;
	if (!b) 
	  return 1;

	for (; *a && *b; a++, b++) 
	{
		if (*a != *b) 
		  return *a - *b;
  }
  
	if (*a) 
	  return 1;
	if (*b) 
	  return -1;
	return 0;
}

int16 memcmp(const uint8 *a, const uint8 *b, uint32 n)
{
	uint32 i;

	if (!a && !b) 
	  return 0;
	if (!a) 
	  return -1;
	if (!b) 
	  return 1;

	for (i=0; i<n; i++)
	{
		if (a[i] != b[i]) 
		  return a[i] - b[i];
  }

	return 0;
}

uint32 strlen(const char *src)
{
	uint32 i;

	if (!src)
	  return 0;
	for (i=0; src[i]; i++);
	return i;
}

char *strcat(char *dst, const char *src)
{
  int32 i=0, j=0;
  
	if (!src || !dst) 
	  return dst;

	while (dst[i])
	  ++i;
	while (src[j])
		dst[i++]=src[j++];
  dst[i] = '\0';
  
	return dst;
}

void *operator new(unsigned long size)
{
	ArmletData *ad=(ArmletData *)GetR10();
	void *retval;

	PutR10(ad->r10org);
	ad->r9curr=GetR9();
	PutR9(ad->r9org);
	retval = (void *)ad->call68k(ad->emulStateP, (uint32)ad->malloc, &size, 4 | kPceNativeWantA0);
	ad->r10org = GetR10();
	PutR10((uint32)ad);
	ad->r9org = GetR9();
  PutR9(ad->r9curr);

	return retval;
}

void operator delete(void *p)
{
	ArmletData *ad=(ArmletData *)GetR10();

	PutR10(ad->r10org);
	ad->r9curr=GetR9();
	PutR9(ad->r9org);
	ad->call68k(ad->emulStateP, (uint32)ad->free, &p, 4);
	ad->r10org = GetR10();
	PutR10((uint32)ad);
	ad->r9org = GetR9();
	PutR9(ad->r9curr);
}
void *operator new[](unsigned long size)
{ 
  return ::operator new(size); 
}
void operator delete[](void *p)
{
  ::operator delete(p);
}
